/*!
 * MyHero
 * Copyright(c) 2018 xiadi
 * MIT Licensed
 */

/**
 * Module dependencies.
 */
var fs = require('fs');
var path = require('path');
var application = require('./application');
var Package = require('../package');

/**
 * Expose `createApplication()`.
 *
 * @module
 */

var MyHero = module.exports = {};

/**
 * Framework version.
 */

MyHero.version = Package.version;

/**
 * Event definitions that would be emitted by app.event
 */
MyHero.events = require('./util/events');

/**
 * auto loaded components
 */
MyHero.components = {};

/**
 * auto loaded filters
 */
MyHero.filters = {};

/**
 * auto loaded rpc filters
 */
MyHero.rpcFilters = {};

/**
 * connectors
 */
MyHero.connectors = {};
MyHero.connectors.__defineGetter__('sioconnector', load.bind(null, './connectors/sioconnector'));
MyHero.connectors.__defineGetter__('hybridconnector', load.bind(null, './connectors/hybridconnector'));
MyHero.connectors.__defineGetter__('udpconnector', load.bind(null, './connectors/udpconnector'));
MyHero.connectors.__defineGetter__('mqttconnector', load.bind(null, './connectors/mqttconnector'));

/**
 * pushSchedulers
 */
MyHero.pushSchedulers = {};
MyHero.pushSchedulers.__defineGetter__('direct', load.bind(null, './pushSchedulers/direct'));
MyHero.pushSchedulers.__defineGetter__('buffer', load.bind(null, './pushSchedulers/buffer'));

var self = this;

/**
 * Create an myhero application.
 *
 * @return {Application}
 * @api public
 */
MyHero.createApp = function (opts) {
  var app = application;
  app.init(opts);
  self.app = app;
  return app;
};

/**
 * Get application
 */
Object.defineProperty(MyHero, 'app', {
  get:function () {
    return self.app;
  }
});

/**
 * Auto-load bundled components with getters.
 */
fs.readdirSync(__dirname + '/components').forEach(function (filename) {
  if (!/\.js$/.test(filename)) {
    return;
  }
  var name = path.basename(filename, '.js');
  var _load = load.bind(null, './components/', name);
  
  MyHero.components.__defineGetter__(name, _load);
  MyHero.__defineGetter__(name, _load);
});

fs.readdirSync(__dirname + '/filters/handler').forEach(function (filename) {
  if (!/\.js$/.test(filename)) {
    return;
  }
  var name = path.basename(filename, '.js');
  var _load = load.bind(null, './filters/handler/', name);
  
  MyHero.filters.__defineGetter__(name, _load);
  MyHero.__defineGetter__(name, _load);
});

fs.readdirSync(__dirname + '/filters/rpc').forEach(function (filename) {
  if (!/\.js$/.test(filename)) {
    return;
  }
  var name = path.basename(filename, '.js');
  var _load = load.bind(null, './filters/rpc/', name);
  
  MyHero.rpcFilters.__defineGetter__(name, _load);
});

function load(path, name) {
  if (name) {
    return require(path + name);
  }
  return require(path);
}
